
// symbreg.h

/* ---------------------------------------------------------------

Symbolic Regression

An example for how to use gpc++ - The Genetic Programming Kernel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

Copyright 1996, 1997 Thomas Weinbrenner

For comments, improvements, additions (or even money !?) contact:

Thomas Weinbrenner
Grauensteinstr. 26
35789 Laimbach
Germany
E-mail: thomasw@emk.e-technik.th-darmstadt.de
WWW:    http://www.emk.e-technik.th-darmstadt/~thomasw

--------------------------------------------------------------- */

#define DEF_TUPLE_SIZE     3

// We are now inheriting the classes we want to change. These are the
// three classes GPGene, GP and GPPopulation.  Read the documentation
// for what has to be done here.  Remember it makes not a lot of sense
// to implement Load/Save here, otherwise we should definitely have to
// provide for the isA(), load(), save() and createObject() functions.

class MyGP;
class tuple;

class MyGene : public GPGene
{
public:
  MyGene (GPNode& gpo) : GPGene (gpo) {}

  MyGene (const MyGene& gpo) : GPGene (gpo) { }
  virtual GPObject& duplicate () { return *(new MyGene(*this)); }
  virtual GPGene* createChild (GPNode& gpo) {
    return new MyGene (gpo); }

  virtual void printOn (ostream& os);
  void printMathStyle (ostream& os, int lastPrecedence=0);
  void printTeXStyle (ostream& os, int lastPrecedence=0);

  MyGene* NthMyChild (int n) {
    return (MyGene*) GPContainer::Nth (n); }

  double evaluate (tuple &x, MyGP& gp);
};



class MyGP : public GP 
{
public:
  MyGP (int genes) : GP (genes) {}

  MyGP (MyGP& gpo) : GP (gpo) { }
  virtual GPObject& duplicate () { return *(new MyGP(*this)); }
  virtual GPGene* createGene (GPNode& gpo) {
    return new MyGene (gpo); }

  virtual void printOn (ostream& os);

  MyGene* NthMyGene (int n) {
    return (MyGene*) GPContainer::Nth (n); }
  virtual void evaluate ();
  virtual void scoreOnTest(float& fp, float& m, float &fa, float &h, double &f);
};



class MyPopulation : public GPPopulation
{
public:
  MyPopulation (GPVariables& GPVar_, GPAdfNodeSet& adfNs_) : 
    GPPopulation (GPVar_, adfNs_) {}

  MyPopulation (MyPopulation& gpo) : GPPopulation(gpo) {}
  virtual GPObject& duplicate () { return *(new MyPopulation(*this)); }

  // Don t check for ultimate diversity as it takes a long time.
  // Accept every created GP
  virtual int checkForValidCreation (GP&) { return 1; }

  virtual GP* createGP (int numOfGenes) { return new MyGP (numOfGenes); }
  /* Function to access the stdFitness varibale of GPPopulation */
  virtual MyGP* NthMyGP (int n) { return (MyGP*) GPContainer::Nth (n); }
  virtual void evaluateGeneration(int printLegend, int generation,
				  ostream& fout, ostream& bout);
};

/* Lets make a class for the tuples that we need */

class tuple {
 public:
  /* Types */
  enum category {ANOMOLY=0, OK};
  category tclass;

  /* Function to do the translation */
  char convertToNumber(char c){
    switch (c) {
    case 'A':
      return -3;
    case 'B':
      return -2;
    case 'C':
      return -1;
    case 'D':
      return 1;
    case 'E':
      return 2;
    case 'F':
      return 3;
    default:
      cout << " Wrong symbol" << endl;
      return -3;
    }
  }

  /* Constructor */
  tuple(int n=DEF_TUPLE_SIZE) {
    size = n;
    val = new int[size];
    for (int i=0; i<size; i++)
      val[i] = 0;
  }
  tuple(tuple &t) {
    size = t.size;
    val = new int[size];
    for (int i=0; i<size; i++)
      val[i] = t.val[i];
  }
  /* Destructor */
  ~tuple() { if (val) delete [] val; }
  /* Other functions */
  int getValue(int n){
    int index = -1;
    if (n <= size && n >0)
      index = n - 1;
    if (n < 0)
      /* Check this return val at the call point!! */
      return 0;
    return val[index];
  }

  int makeTuple(int n=DEF_TUPLE_SIZE, ifstream &fin) {
    createTuple(n);
    return readTuple(fin);
  }

  void createTuple(int n=DEF_TUPLE_SIZE) {
    size = n;
    if (val)
      delete [] val;
    val = new int[size];
    for (int i=0; i<size; i++)
      val[i] = 0;
  }
  int readTuple(ifstream &fin) {
    char c;
    if (!fin)
      return (0);
    for (int i =0; i< size; i++) {
      if (fin.eof())
	/* Stopped in the middle....*/
	return 0;
      fin >> c;
      if (c == '\0')
	return (1);
	val[i] = convertToNumber(c);
    }
    fin >> c;
    if (c == 'y')
      tclass = ANOMOLY;
    else
      tclass = OK;
    /* Gobble the ';" */
    fin >> c;
  }
 protected:
  int *val, size;
};
